@@ -20,6 +20,8 @@ usage: cb-collect -h | --help
        cb-collect --project=<github_project> --ssh-pkey=<ssh_pkey_file>
            [--ssh-user=<user>]
            [--timeout=<time_sec>]
+           [--update-interval=<time_sec>]
+           [--runners=<runner_count>]
 
 options:
     --project=<github_project>
@@ -35,11 +37,20 @@ options:
     --timeout=<time_sec>
         Wait time_sec seconds of inactivity on the message
         broker before return. Default: 30sec
+
+    --update-interval=<time_sec>
+        Update interval to ask for new packages/images
+        Default: 30sec
+
+    --runners=<runner_count>
+        Number of runners in the cluster
 """
 import os
 import time
 import yaml
-import threading
+from threading import (
+    Thread, Lock
+)
 from datetime import datetime
 from docopt import docopt
 from cloud_builder.version import __version__
@@ -60,7 +71,8 @@ from typing import (
 repo_metadata = NamedTuple(
     'repo_metadata', [
         ('repo_type', str),
-        ('repo_file', str)
+        ('repo_file', str),
+        ('repo_path', str)
     ]
 )
 
@@ -112,6 +124,8 @@ def main() -> None:
     build_repos(
         broker, int(args['--timeout'] or 30),
         args['--ssh-pkey'], args['--ssh-user'] or 'ec2-user',
+        int(args['--update-interval'] or 30),
+        int(args['--runners'] or 0),
         log
     )
 
@@ -203,7 +217,7 @@ def send_project_info_requests(
 
 def group_info_response(
     broker: Any, request_id_list: List[str], timeout_sec: int,
-    log: CBCloudLogger
+    runner_count: int, log: CBCloudLogger
 ) -> Dict:
     """
     Watch on the info_response queue for information
@@ -213,6 +227,8 @@ def group_info_response(
     :param Any broker: Instance of broker factory
     :param List request_id_list: list of matching request IDs
     :param int timeout: read timeout in sec on cb-info response
+    :param int runner_count: number of runners
+    :param CBCloudLogger log: logger object
 
     :return:
         Dictionary of IP groups and their data to fetch
@@ -231,12 +247,16 @@ def group_info_response(
 
     :rtype: Dict
     """
+    stop_reading_at = len(request_id_list) * runner_count
+    response_count = 0
     info_records: Dict[str, List] = {}
     # Read package info responses and group them by a unique id
     # consisting out of: project-package-arch-dist information
     try:
         timeout_loop_start = time.time()
-        while time.time() < timeout_loop_start + timeout_sec + 1:
+        while time.time() < timeout_loop_start + timeout_sec + 1 and (
+            stop_reading_at == 0 or stop_reading_at > response_count
+        ):
             message = None
             for message in broker.read(
                 topic=Defaults.get_info_response_queue_name(),
@@ -248,29 +268,45 @@ def group_info_response(
                 if response:
                     broker.acknowledge()
                     if response['request_id'] in request_id_list:
-                        project_id = ''
-                        if 'package' in response:
-                            project_id = '{0}_{1}_{2}'.format(
-                                response['project'],
-                                response['package']['arch'],
-                                response['package']['dist']
-                            )
-                        elif 'image' in response:
-                            project_id = '{0}_{1}_{2}'.format(
-                                response['project'],
-                                response['image']['arch'],
-                                'image'
-                            )
-                        if project_id and project_id in info_records:
-                            info_records[project_id].append(response)
-                        elif project_id:
-                            info_records[project_id] = [response]
+                        if response['utc_modification_time'] != 'none':
+                            # A utc_modification_time set to none, indicates
+                            # that this info response was sent from an info
+                            # service which has no data for the requested
+                            # package/image (info server configured with
+                            # --always-respond). These responses we ignore for
+                            # the collector, but they are important to identify
+                            # the time when all runners have responded.
+                            project_id = ''
+                            if 'package' in response:
+                                project_id = '{0}_{1}_{2}'.format(
+                                    response['project'],
+                                    response['package']['arch'],
+                                    response['package']['dist']
+                                )
+                            elif 'image' in response:
+                                project_id = '{0}_{1}_{2}'.format(
+                                    response['project'],
+                                    response['image']['arch'],
+                                    'image'
+                                )
+                            if project_id and project_id in info_records:
+                                info_records[project_id].append(response)
+                            elif project_id:
+                                info_records[project_id] = [response]
+                        response_count += 1
             if not message:
                 break
     except Exception as issue:
         log.error(format(issue))
         return {}
 
+    if stop_reading_at > 0 and response_count != stop_reading_at:
+        log.warning(
+            'runner count set to {0} but got {1} responses'.format(
+                stop_reading_at, response_count
+            )
+        )
+
     # Walk through project_id grouped responses and take out the
     # latest available build. Group all data by project_id/source_ip
     runner_responses: Dict[str, Dict[str, List]] = {}
@@ -325,7 +361,7 @@ def group_info_response(
 
 def build_repos(
     broker: Any, timeout: int, ssh_pkey_file: str, user: str,
-    log: CBCloudLogger
+    update_interval: int, runner_count: int, log: CBCloudLogger
 ) -> None:
     """
     Application loop - building project repositories
@@ -333,6 +369,8 @@ def build_repos(
     :param Any broker: Instance of broker factory
     :param int timeout: Read timeout on info response queue
     """
+    thread_lock = Lock()
+
     while(True):
         update_project()
         projects_tree = build_project_tree()
@@ -340,22 +378,27 @@ def build_repos(
             broker, projects_tree, log
         )
         runner_responses = group_info_response(
-            broker, request_id_list, timeout, log
+            broker, request_id_list, timeout, runner_count, log
         )
         if not runner_responses:
-            log.info(f'No runners responded... sleeping {timeout} sec')
-            time.sleep(timeout)
+            log.info(
+                f'No runners responded... sleeping {update_interval} sec'
+            )
+            time.sleep(update_interval)
             continue
         for project_id in runner_responses.keys():
-            project_repo_thread = threading.Thread(
+            project_repo_thread = Thread(
                 target=build_project_repo,
                 args=(
                     project_id, runner_responses[project_id],
-                    ssh_pkey_file, user, log
+                    ssh_pkey_file, user, thread_lock, log
                 )
             )
             project_repo_thread.start()
 
+        # wait update_interval seconds before next round
+        time.sleep(update_interval)
+
 
 def cleanup_project_repo(
     repo_project_path: str, log: CBCloudLogger
@@ -474,9 +517,9 @@ def cleanup_project_repo_packages_targets(
 
 def build_project_repo(
     project_id: str, runner_responses_for_project: Dict,
-    ssh_pkey_file: str, user: str, log: CBCloudLogger
+    ssh_pkey_file: str, user: str, thread_lock: Lock, log: CBCloudLogger
 ) -> None:
-    if not _set_lock(project_id):
+    if not _set_lock(project_id, thread_lock):
         log.info(f'Repo sync for {project_id} is locked')
         return
     target_path = os.path.normpath(
@@ -493,6 +536,7 @@ def build_project_repo(
 
     if not os.path.exists(target_path):
         Path.create(target_path)
+    sync_files: Dict[str, List[str]] = {}
     for source_ip in runner_responses_for_project.keys():
         for source_spec in runner_responses_for_project[source_ip]:
             (source_file, project_package_name) = source_spec.split('|')
@@ -507,17 +551,24 @@ def build_project_repo(
                     package_name=project_package_name,
                     repo_file=repo_meta.repo_file
                 )
-                sync_call = Command.run(
-                    [
-                        'rsync', '-av', '-e', 'ssh -i {0} -o {1}'.format(
-                            ssh_pkey_file, 'StrictHostKeyChecking=accept-new'
-                        ), remote_source_file, repo_meta.repo_file
-                    ], raise_on_error=False
+                if repo_meta.repo_path in sync_files:
+                    sync_files[repo_meta.repo_path].append(remote_source_file)
+                else:
+                    sync_files[repo_meta.repo_path] = [remote_source_file]
+    for repo_path in sorted(sync_files.keys()):
+        sync_call = Command.run(
+            [
+                'rsync', '-av', '-e', 'ssh -i {0} -o {1}'.format(
+                    ssh_pkey_file, 'StrictHostKeyChecking=accept-new'
                 )
-                if sync_call.output:
-                    log.info(sync_call.output)
-                if sync_call.error:
-                    log.error(sync_call.error)
+            ] + sync_files[repo_path] + [
+                repo_meta.repo_path
+            ], raise_on_error=False
+        )
+        if sync_call.output:
+            log.info(sync_call.output)
+        if sync_call.error:
+            log.error(sync_call.error)
 
     log.info(f'Running cleanup for: {project_path!r}')
     # check if project got deleted from source
@@ -575,7 +626,7 @@ def _update_project_indicator(
 def _get_repo_path_for_binary(
     binary_file_name: str, target_path: str, log: CBCloudLogger
 ) -> repo_metadata:
-    repo_path = None
+    repo_path = 'unknown'
     repo_type = 'unknown'
     if binary_file_name.endswith('.src.rpm'):
         repo_path = f'{target_path}/src'
@@ -593,12 +644,14 @@ def _get_repo_path_for_binary(
         )
         return repo_metadata(
             repo_type=repo_type,
+            repo_path=repo_path,
             repo_file=''
         )
     if not os.path.isdir(repo_path):
         Path.create(repo_path)
     return repo_metadata(
         repo_type=repo_type,
+        repo_path=repo_path,
         repo_file=os.sep.join(
             [repo_path, os.path.basename(binary_file_name)]
         )
@@ -628,12 +681,16 @@ def _create_rpm_repo(target_path: str, log: CBCloudLogger) -> None:
     # - signing of package files via rpmsign
 
 
-def _set_lock(project_id: str) -> bool:
+def _set_lock(project_id: str, thread_lock: Lock) -> bool:
     """
-    Create lock file for the given project path
-    Returns False if lock is already present
+    Create lock file for the given project path. Returns
+    False if lock is already present. During the creation
+    of the file based lock a mutex lock is set to ensure
+    no other thread interferes with the file lock for the
+    project path.
 
     :param str project_id: unique path name to describe project
+    :param Lock lock: Mutex Lock
 
     :return:
         Bool value indicating if lock is set or not.
@@ -641,15 +698,19 @@ def _set_lock(project_id: str) -> bool:
 
     :rtype: bool
     """
+    thread_lock.acquire()
+    lock_set_action = False
     lock_file = '/var/lock/{0}.lock'.format(
         project_id.replace(os.sep, '_')
     )
     if os.path.isfile(lock_file):
-        return False
+        lock_set_action = False
     else:
         with open(lock_file, 'w'):
             pass
-    return True
+        lock_set_action = True
+    thread_lock.release()
+    return lock_set_action
 
 
 def _set_free(project_id: str) -> None:
